@@ -1,5 +1,6 @@
 import json as complexjson
 import numbers
+import warnings
 from typing import Any, Dict, Generator, List, Union
 
 from requests.exceptions import ChunkedEncodingError
@@ -13,6 +14,7 @@ from cognite.client.data_classes.geospatial import (
     FeatureList,
     FeatureType,
     FeatureTypeList,
+    FeatureTypePatch,
     FeatureTypeUpdate,
     OrderSpec,
 )
@@ -47,7 +49,11 @@ class GeospatialAPI(APIClient):
                 >>> c = CogniteClient()
                 >>> feature_types = [
                 ...     FeatureType(external_id="wells", properties={"location": {"type": "POINT", "srid": 4326}})
-                ...     FeatureType(external_id="pipelines", properties={"location": {"type": "LINESTRING", "srid": 2001}})
+                ...     FeatureType(
+                ...       external_id="cities",
+                ...       properties={"name": {"type": "STRING", "size": 10}},
+                ...       search_spec={"name_index": {"properties": ["name"]}}
+                ...     )
                 ... ]
                 >>> res = c.geospatial.create_feature_types(feature_types)
         """
@@ -72,7 +78,7 @@ class GeospatialAPI(APIClient):
 
                 >>> from cognite.client import CogniteClient
                 >>> c = CogniteClient()
-                >>> c.geospatial.delete_feature_types(external_id=["wells", "pipelines"])
+                >>> c.geospatial.delete_feature_types(external_id=["wells", "cities"])
         """
         extra_body_fields = {"recursive": True} if recursive else {}
         return self._delete_multiple(
@@ -126,11 +132,11 @@ class GeospatialAPI(APIClient):
         )
 
     def update_feature_types(self, update: Union[FeatureTypeUpdate, List[FeatureTypeUpdate]] = None) -> FeatureTypeList:
-        """`Patch feature types`
+        """`Update feature types (Deprecated)`
         <https://docs.cognite.com/api/v1/#operation/updateFeatureTypes>
 
         Args:
-            update (Union[FeatureTypeUpdate, List[FeatureTypePatch]]): the update to apply
+            update (Union[FeatureTypeUpdate, List[FeatureTypeUpdate]]): the update to apply
 
         Returns:
             FeatureTypeList: The updated feature types.
@@ -142,8 +148,11 @@ class GeospatialAPI(APIClient):
                 >>> from cognite.client.data_classes.geospatial import PropertyAndSearchSpec
                 >>> from cognite.client import CogniteClient
                 >>> c = CogniteClient()
-                >>> res = c.geospatial.update_feature_types(update=FeatureTypeUpdate(external_id="wells", add=PropertyAndSearchSpec(properties={"altitude": {"type": "DOUBLE"}}, search_spec={"altitude_idx": {"properties": ["altitude"]}})))
+                >>> res = c.geospatial.update_feature_types(update=FeatureTypeUpdate(external_id="wells",
+                ...         add=PropertyAndSearchSpec(properties={"altitude": {"type": "DOUBLE"}},
+                ...         search_spec={"altitude_idx": {"properties": ["altitude"]}})))
         """
+        warnings.warn("update_feature_types is deprecated, use patch_feature_types instead.", DeprecationWarning)
         if isinstance(update, FeatureTypeUpdate):
             update = [update]
 
@@ -160,22 +169,80 @@ class GeospatialAPI(APIClient):
         res = self._post(url_path=f"{self._RESOURCE_PATH}/featuretypes/update", json=json)
         return FeatureTypeList._load(res.json()["items"], cognite_client=self._cognite_client)
 
+    def patch_feature_types(self, patch: Union[FeatureTypePatch, List[FeatureTypePatch]] = None) -> FeatureTypeList:
+        """`Patch feature types`
+        <https://docs.cognite.com/api/v1/#operation/updateFeatureTypes>
+
+        Args:
+            patch (Union[FeatureTypePatch, List[FeatureTypePatch]]): the patch to apply
+
+        Returns:
+            FeatureTypeList: The patched feature types.
+
+        Examples:
+
+            Add one property to a feature type and add indexes
+
+                >>> from cognite.client.data_classes.geospatial import Patches
+                >>> from cognite.client import CogniteClient
+                >>> c = CogniteClient()
+                >>> res = c.geospatial.patch_feature_types(
+                ...    patch=FeatureTypePatch(
+                ...       external_id="wells",
+                ...       property_patches=Patches(add={"altitude": {"type": "DOUBLE"}}),
+                ...       search_spec_patches=Patches(
+                ...         add={
+                ...           "altitude_idx": {"properties": ["altitude"]},
+                ...           "composite_idx": {"properties": ["location", "altitude"]}
+                ...         }
+                ...       )
+                ...    )
+                ... )
+
+            Add an additional index to an existing property
+
+                >>> from cognite.client.data_classes.geospatial import Patches
+                >>> from cognite.client import CogniteClient
+                >>> c = CogniteClient()
+                >>> res = c.geospatial.patch_feature_types(
+                ...    patch=FeatureTypePatch(
+                ...         external_id="wells",
+                ...         search_spec_patches=Patches(add={"location_idx": {"properties": ["location"]}})
+                ... ))
+
+        """
+        if isinstance(patch, FeatureTypePatch):
+            patch = [patch]
+        json = {
+            "items": [
+                {
+                    "externalId": it.external_id,
+                    "update": {"properties": it.property_patches, "searchSpec": it.search_spec_patches},
+                }
+                for it in patch
+            ]
+        }
+        res = self._post(url_path=f"{self._RESOURCE_PATH}/featuretypes/update", json=json)
+        return FeatureTypeList._load(res.json()["items"], cognite_client=self._cognite_client)
+
     def create_features(
         self,
         feature_type_external_id: str,
         feature: Union[Feature, List[Feature], FeatureList],
         allow_crs_transformation: bool = False,
+        chunk_size: int = None,
     ) -> Union[Feature, FeatureList]:
         """`Creates features`
         <https://docs.cognite.com/api/v1/#operation/createFeatures>
 
         Args:
-            feature_type_external_id : Feature type definition for the features to create.
+            feature_type_external_id: Feature type definition for the features to create.
             feature: one feature or a list of features to create or a FeatureList object
             allow_crs_transformation: If true, then input geometries will be transformed into the Coordinate Reference
                 System defined in the feature type specification. When it is false, then requests with geometries in
                 Coordinate Reference System different from the ones defined in the feature type will result in
                 CogniteAPIError exception.
+            chunk_size: maximum number of items in a single request to the api
 
         Returns:
             Union[Feature, FeatureList]: Created features
@@ -192,12 +259,18 @@ class GeospatialAPI(APIClient):
                 >>> res = c.geospatial.create_feature_types(feature_types)
                 >>> res = c.geospatial.create_features("my_feature_type", Feature(external_id="my_feature", location={"wkt": "POINT(1 1)"}, temperature=12.4))
         """
+        if chunk_size is not None and (chunk_size < 1 or chunk_size > self._CREATE_LIMIT):
+            raise ValueError(f"The chunk_size must be strictly positive and not exceed {self._CREATE_LIMIT}")
         if isinstance(feature, FeatureList):
             feature = list(feature)
         resource_path = self._feature_resource_path(feature_type_external_id)
         extra_body_fields = {"allowCrsTransformation": "true"} if allow_crs_transformation else {}
         return self._create_multiple(
-            items=feature, resource_path=resource_path, cls=FeatureList, extra_body_fields=extra_body_fields
+            items=feature,
+            resource_path=resource_path,
+            cls=FeatureList,
+            extra_body_fields=extra_body_fields,
+            limit=chunk_size,
         )
 
     def delete_features(self, feature_type_external_id: str, external_id: Union[str, List[str]] = None) -> None:
@@ -217,13 +290,16 @@ class GeospatialAPI(APIClient):
 
                 >>> from cognite.client import CogniteClient
                 >>> c = CogniteClient()
-                >>> c.geospatial.delete_feature("my_feature_type", external_id=my_feature)
+                >>> c.geospatial.delete_features("my_feature_type", external_id=my_feature)
         """
         resource_path = self._feature_resource_path(feature_type_external_id)
         self._delete_multiple(external_ids=external_id, wrap_ids=True, resource_path=resource_path)
 
     def retrieve_features(
-        self, feature_type_external_id: str, external_id: Union[str, List[str]] = None
+        self,
+        feature_type_external_id: str,
+        external_id: Union[str, List[str]] = None,
+        properties: Dict[str, Any] = None,
     ) -> FeatureList:
         """`Retrieve features`
         <https://docs.cognite.com/api/v1/#operation/getFeaturesByIds>
@@ -231,6 +307,7 @@ class GeospatialAPI(APIClient):
         Args:
             feature_type_external_id : Feature type external id for the features to retrieve.
             external_id (Union[str, List[str]]): External ID or list of external ids
+            properties (Dict[str, Any]): the output property selection
 
         Returns:
             FeatureList: Requested features or None if it does not exist.
@@ -241,11 +318,15 @@ class GeospatialAPI(APIClient):
 
                 >>> from cognite.client import CogniteClient
                 >>> c = CogniteClient()
-                >>> c.geospatial.retrieve_feature("my_feature_type", external_id="my_feature")
+                >>> c.geospatial.retrieve_features("my_feature_type", external_id="my_feature")
         """
         resource_path = self._feature_resource_path(feature_type_external_id)
         return self._retrieve_multiple(
-            wrap_ids=True, external_ids=external_id, resource_path=resource_path, cls=FeatureList
+            wrap_ids=True,
+            external_ids=external_id,
+            resource_path=resource_path,
+            other_params={"output": {"properties": properties}},
+            cls=FeatureList,
         )
 
     def update_features(
@@ -253,6 +334,7 @@ class GeospatialAPI(APIClient):
         feature_type_external_id: str,
         feature: Union[Feature, List[Feature]],
         allow_crs_transformation: bool = False,
+        chunk_size: int = None,
     ) -> FeatureList:
         """`Update features`
         <https://docs.cognite.com/api/v1/#operation/updateFeatures>
@@ -264,6 +346,7 @@ class GeospatialAPI(APIClient):
                 System defined in the feature type specification. When it is false, then requests with geometries in
                 Coordinate Reference System different from the ones defined in the feature type will result in
                 CogniteAPIError exception.
+            chunk_size: maximum number of items in a single request to the api
 
         Returns:
             FeatureList: Updated features
@@ -278,14 +361,16 @@ class GeospatialAPI(APIClient):
                 >>> # do some stuff
                 >>> my_updated_feature = c.geospatial.update_features("my_feature_type", Feature(external_id="my_feature", temperature=6.237))
         """
+        if chunk_size is not None and (chunk_size < 1 or chunk_size > self._UPDATE_LIMIT):
+            raise ValueError(f"The chunk_size must be strictly positive and not exceed {self._UPDATE_LIMIT}")
         if isinstance(feature, FeatureList):
             feature = list(feature)
         # updates for feature are not following the patch structure from other resources
-        # they are more like a replace so an update looks like a feature creation (yeah, borderline ?)
+        # they are more like a replace so an update looks like a feature creation
         resource_path = self._feature_resource_path(feature_type_external_id) + "/update"
         extra_body_fields = {"allowCrsTransformation": "true"} if allow_crs_transformation else {}
         return self._create_multiple(
-            feature, resource_path=resource_path, cls=FeatureList, extra_body_fields=extra_body_fields
+            feature, resource_path=resource_path, cls=FeatureList, extra_body_fields=extra_body_fields, limit=chunk_size
         )
 
     def search_features(
